@@ -15,9 +15,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+   USA.  */
 
-#include "silo.h"
+#include <silo.h>
 #include <asm/mostek.h>
 
 struct sun4m_timer_regs {
@@ -157,12 +158,12 @@ static inline int sun4u_init_timer ()
         clock_frequency = prom_getint(prom_root_node, "clock-frequency") / 100;
     if (notimer) {
         sun4u_notimer = 1;
-        __asm__ __volatile__ ("
-        	rd	%%tick_cmpr, %%g1
-        	stx	%%g1, [%0]
-        	mov	1, %%g1
-        	sllx    %%g1, 63, %%g1
-        	wr      %%g1, 0, %%tick_cmpr"
+        __asm__ __volatile__ ("\t"
+        	"rd	%%tick_cmpr, %%g1\n\t"
+        	"stx	%%g1, [%0]\n\t"
+        	"mov	1, %%g1\n\t"
+        	"sllx    %%g1, 63, %%g1\n\t"
+        	"wr      %%g1, 0, %%tick_cmpr"
         	: : "r" (&sun4u_tickcmpr) : "g1");
     }
     return 0;
@@ -206,10 +207,9 @@ int init_timer ()
 void close_timer ()
 {
     if (sun4u_notimer) {
-        __asm__ __volatile__("
-        	ldx	[%0], %%g1
-        	wrpr	%%g0, 0, %%tick
-        	wr	%%g1, 0, %%tick_cmpr"
+        __asm__ __volatile__("\t"
+        	"ldx	[%0], %%g1\n\t"
+        	"wr	%%g1, 0, %%tick_cmpr"
         	: : "r" (&sun4u_tickcmpr) : "g1");
     }
     if (addr_to_free) {
@@ -227,10 +227,10 @@ void reset_ticks (void)
 {
     ticks = 0;
     if (architecture == sun4u) {
-        __asm__ __volatile__ ("
-        	rd	%%tick, %%g1
-        	stx	%%g1, [%0]
-        	" : : "r" (&ticks) : "g1");
+        __asm__ __volatile__ ("\t"
+        	"rd	%%tick, %%g1\n\t"
+        	"stx	%%g1, [%0]\n\t"
+        	: : "r" (&ticks) : "g1");
     }
 }
 
@@ -268,9 +268,15 @@ int get_ticks (void)
     switch (architecture) {
         case sun4c: i = sun4c_timer->count10; break;
         case sun4m: i = sun4m_timer->count; break;
-        case sun4p: i = sun4p_lda(PCIC_PHYSADDR+PCIC_SYS_COUNT);
-                    i >>= 2;  /* PROLL does this and its timing is sure right */
-                    break;
+        case sun4p:
+		i = sun4p_lda(PCIC_PHYSADDR+PCIC_SYS_COUNT) & 0x7FFFFFFF;
+		if (i >= lasti)
+			ticks += i - lasti;
+		else
+			ticks += (0x7FFFFFFF - lasti) + i;
+		lasti = i;
+		/* 1 increment every 4 CPU clocks (@ 100MHz) */
+		return (int) (ticks / 250000);
         case sun4d: /* I cannot get the normal sun4d timer working
 		       during bootstrapping, so unfortunately I can give
 		       just a 1000ms precision. */
@@ -282,12 +288,12 @@ int get_ticks (void)
 		    if (i <= lasti) return 0;
 		    return i - lasti;
         case sun4u: 
-        	__asm__ __volatile__ ("
-        		ldx	[%2], %%g2
-        		rd	%%tick, %%g1
-        		sub	%%g1, %%g2, %%g1
-        		udivx	%%g1, %1, %0
-        	" : "=r" (i) : "r" (clock_frequency), "r" (&ticks) : "g1", "g2");
+        	__asm__ __volatile__ ("\t"
+        		"ldx	[%2], %%g2\n\t"
+        		"rd	%%tick, %%g1\n\t"
+        		"sub	%%g1, %%g2, %%g1\n\t"
+        		"udivx	%%g1, %1, %0\n\t"
+		: "=r" (i) : "r" (clock_frequency), "r" (&ticks) : "g1", "g2");
         	return i;
         default: return 0;
     }
